﻿#region using
using System;
using System.Text;
using System.Data;
using System.Web;
using System.Web.UI;
using BlogEngine.Core;
using BlogEngine.Core.Web;
using BlogEngine.Core.Web.Controls;
#endregion

/// <summary>
/// A simple wrapper for JS SyntaxHighlighter <http://code.google.com/p/syntaxhighlighter/>
/// </summary>
[Extension("JSSyntaxHighlighter Library Wrapper", "1.0", "<a href=\"http://www.nurasto.com/\">Dityo Nurasto</a>")]
public class JSSyntaxHighlighter
{
    //Constant setting
    static protected ExtensionSettings _settings = null;
    private const string shPath = "SyntaxHighlighter/";

    //get the root of the BlogEngine.Net
    private string blogRoot = null;
    

    #region Constructor
    public JSSyntaxHighlighter()
    {
        this.blogRoot = Utils.AbsoluteWebRoot.ToString();

        //need to mention the namespace cause by ambiguity
        BlogEngine.Core.Page.Serving += new EventHandler<ServingEventArgs>(Item_Serving);
        Post.Serving += new EventHandler<ServingEventArgs>(Item_Serving);

        //Extension Settings Intialization
        ExtensionSettings settings = new ExtensionSettings("JSSyntaxHighlighter");
        settings.IsScalar = false;

        settings.AddParameter("syntax", "高亮语法名称", 10, true);
        settings.AddParameter("file", "高亮脚本文件名", 75, true);

        settings.Help = "You may add the Syntax Highlighter file you need.";
        //by Spoony 09.02.06
        settings.Help += "<br />在这里添加您需要高亮处理的语言。进一步了解如何使用请参见<a target=\"_blank\" href=\"http://www.blogyi.net/blog/post/plugins-jssyntaxhighlighter.aspx\">使用说明</a>。";

        //inital value
        settings.AddValues(new string[] { "C#", "shBrushCSharp.js" });

        ExtensionManager.ImportSettings(settings);
        _settings = ExtensionManager.GetSettings("JSSyntaxHighlighter");

    }
    #endregion

    #region EventHandler
    //woho we access the basePage ... thanx to René Kuss, LightBox extension creator
    private void Item_Serving(Object sender, ServingEventArgs e)
    {       
        if (HttpContext.Current.CurrentHandler is BlogBasePage)
            (HttpContext.Current.CurrentHandler as BlogBasePage).LoadComplete += new EventHandler(basePage_LoadComplete);
    }

    //woho we access the basePage ... thanx to René Kuss, LightBox extension creator
    private void basePage_LoadComplete(Object sender, EventArgs e)
    {
        BlogBasePage basePage = sender as BlogBasePage;
        this.injectToHead(basePage);
    }
    #endregion

    #region Method
    private void injectToHead(BlogBasePage blogBasePage)
    {
        LiteralControl shLiteral = new LiteralControl();        

        shLiteral.ID = "shCtrlLiteral";
        if (blogBasePage.Header.FindControl(shLiteral.ID) == null)
        {
            shLiteral.Text = loadSyntaxHighlighter();
            blogBasePage.Header.Controls.Add(shLiteral);
        }

        if(!blogBasePage.ClientScript.IsStartupScriptRegistered(this.GetType(), 
            "RenderSyntaxHighlighter"))        
            blogBasePage.ClientScript.RegisterStartupScript(this.GetType(),
                "RenderSyntaxHighlighter", renderSyntaxHighlighter(), true);
    }

    private string loadSyntaxHighlighter()
    {
        //initiate the string builder
        StringBuilder jsSource = new StringBuilder();

        //initiate the core of Syntax Highlighter and its CSS
        jsSource.AppendFormat(
            "<link type=\"text/css\" rel=\"stylesheet\" href=\"{0}{1}Styles/SyntaxHighlighter.css\" />",
            blogRoot, shPath);

        jsSource.AppendFormat(
            "<script type=\"text/javascript\" src=\"{0}{1}Scripts/shCore.js\"></script>",
            blogRoot, shPath);

        //retrive syntax file from extension setting
        DataTable table = _settings.GetDataTable();
        foreach (DataRow datarow in table.Rows)
        {
            jsSource.AppendFormat(
                "<script type=\"text/javascript\" src=\"{0}{1}Scripts/{2}\"></script>",
                blogRoot, shPath, (string)datarow["file"]);
        }

        return jsSource.ToString();
    }

    private string renderSyntaxHighlighter()
    {
        StringBuilder jsRender = new StringBuilder();
        
        jsRender.Append("window.onload = function () {");
        jsRender.AppendFormat(
            "dp.SyntaxHighlighter.ClipboardSwf = '{0}{1}Scripts/clipboard.swf';",
            blogRoot, shPath);
        jsRender.Append("dp.SyntaxHighlighter.HighlightAll('code'); }");
        

        return jsRender.ToString();
    }
    #endregion
}
