﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.Collections.Specialized;
using BlogEngine.Core;
using BlogEngine.Core.DataStore;

public partial class widgets_Statistics_widget : WidgetBase
{
    private const double RECACHE_TIME = 20;
    private const string CACHE_KEY = "visits";

    public override string Name
    {
        get { return "Statistics"; }
    }

    public override bool IsEditable
    {
        get { return true; }
    }

    public override void LoadWidget()
    {
        RecacheVisits();
        CheckVisit();
        AddTotals();
    }

    private void AddTotals()
    {
        int comments = 0;
        int raters = 0;
        List<Post> posts = Post.Posts.FindAll(delegate(Post p) { return p.IsVisible; });
        List<BlogEngine.Core.Page> pages = BlogEngine.Core.Page.Pages.FindAll(delegate(BlogEngine.Core.Page p) { return p.IsVisible; });
        foreach (Post post in posts)
        {
            comments += post.ApprovedComments.Count;
            raters += post.Raters;
        }

        ltPosts.Text = posts.Count + " 篇文章<br />";
        ltPages.Text = pages.Count + " 个单页<br />";
        if (BlogSettings.Instance.IsCommentsEnabled)
            ltComments.Text = comments + " 条评论<br />";

        if (BlogSettings.Instance.EnableRating)
            ltRaters.Text = raters + " 次评分<br />";

        //visits
        StringDictionary settings = GetSettings();
        if (settings.ContainsKey("visits"))
        {
            ltVisits.Text = settings["visits"] + " 次访问<br />";
        }
        else
        {
            settings["visits"] = "1";
            SaveSettings(settings);
            ltVisits.Text = "1 次访问<br />";
        }

        //datecreated
        if (!settings.ContainsKey("datecreated"))
        {
            settings["datecreated"] = DateTime.Now.ToString();
            SaveSettings(settings);
        }
        ltDateCreated.Text = DateTime.Parse(settings["datecreated"]).ToString("访问统计开始于 yyyy年M月d日<br />");
        ltAverage.Text = "平均日访问 " + int.Parse(settings["visits"]) / (DateDiff(DateTime.Now, DateTime.Parse(settings["datecreated"])) + 1) + " 次<br />";

        //online
        if (HttpRuntime.Cache[CACHE_KEY] != null)
        {
            Dictionary<string, DateTime> vips = (Dictionary<string, DateTime>)HttpRuntime.Cache[CACHE_KEY];
            ltOnline.Text = "当前 " + vips.Count.ToString() + " 人在线";
        }
        else
        {
            ltOnline.Text = "当前 1 人在线";
        }
    }

    private void CheckVisit()
    {
        string visitorIP = Request.UserHostAddress;

        if (HttpRuntime.Cache[CACHE_KEY] != null)
        {
            Dictionary<string, DateTime> vips = (Dictionary<string, DateTime>)HttpRuntime.Cache[CACHE_KEY];
            if (!vips.ContainsKey(visitorIP))
            {
                vips.Add(visitorIP, DateTime.Now);
                IncVisits();
            }
            else
            {
                vips[visitorIP] = DateTime.Now;
            }

            HttpRuntime.Cache[CACHE_KEY] = vips;
        }
        else
        {
            Dictionary<string, DateTime> vips = new Dictionary<string, DateTime>();
            vips.Add(visitorIP, DateTime.Now);

            IncVisits();

            HttpRuntime.Cache[CACHE_KEY] = vips;
        }
    }

    /// <summary>
    /// Saves settings to data store
    /// </summary>
    /// <param name="settings">Settings</param>
    protected void SaveSettings(StringDictionary settings)
    {
        string cacheId = "be_widget_" + WidgetID;

        WidgetSettings ws = new WidgetSettings(WidgetID.ToString());
        ws.SaveSettings(settings);

        Cache[cacheId] = settings;
    }

    private void IncVisits()
    {
        StringDictionary settings = GetSettings();
        if (settings.ContainsKey("visits"))
        {
            settings["visits"] = Convert.ToString(Convert.ToInt32(settings["visits"]) + 1);
        }
        else
        {
            settings["visits"] = "1";
        }

        SaveSettings(settings);
    }

    private void RecacheVisits()
    {
        string visitorIP = Request.UserHostAddress;

        if (HttpRuntime.Cache[CACHE_KEY] != null)
        {
            Dictionary<string, DateTime> vips = (Dictionary<string, DateTime>)HttpRuntime.Cache[CACHE_KEY];
            try
            {
                foreach (string ip in vips.Keys)
                {
                    if (vips[ip].AddMinutes(RECACHE_TIME).CompareTo(DateTime.Now) < 0)
                        vips.Remove(ip);
                }
            }
            catch { }

            HttpRuntime.Cache[CACHE_KEY] = vips;
        }
        else
        {
            Dictionary<string, DateTime> vips = new Dictionary<string, DateTime>();
            vips.Add(visitorIP, DateTime.Now);

            IncVisits();

            HttpRuntime.Cache[CACHE_KEY] = vips;
        }
    }

    private int DateDiff(DateTime DateTime1, DateTime DateTime2)
    {
        TimeSpan ts1 = new TimeSpan(DateTime1.Ticks);
        TimeSpan ts2 = new TimeSpan(DateTime2.Ticks);
        TimeSpan ts = ts1.Subtract(ts2).Duration();
        return ts.Days;
    }
}